@@ -21,10 +21,6 @@ const (
 	// PublicKeysPerSeed define the number of public keys that get pregenerated
 	// for a seed at startup when searching for balances in the blockchain.
 	PublicKeysPerSeed = 2500
-
-	// WalletSeedPreloadDepth is the number of addresses that get automatically
-	// loaded by the wallet at startup.
-	WalletSeedPreloadDepth = 25
 )
 
 var (
@@ -59,6 +55,7 @@ type (
 	// coming from an address and going to the outputs. The fund types are
 	// 'SiacoinInput', 'SiafundInput'.
 	ProcessedInput struct {
+		ParentID       types.OutputID   `json:"parentid"`
 		FundType       types.Specifier  `json:"fundtype"`
 		WalletAddress  bool             `json:"walletaddress"`
 		RelatedAddress types.UnlockHash `json:"relatedaddress"`
@@ -78,6 +75,7 @@ type (
 	// available. SiacoinInputs and SiafundInputs become available immediately.
 	// ClaimInputs and MinerPayouts become available after 144 confirmations.
 	ProcessedOutput struct {
+		ID             types.OutputID    `json:"id"`
 		FundType       types.Specifier   `json:"fundtype"`
 		MaturityHeight types.BlockHeight `json:"maturityheight"`
 		WalletAddress  bool              `json:"walletaddress"`
@@ -229,11 +227,22 @@ type (
 		// a different directory or deleted.
 		Encrypt(masterKey crypto.TwofishKey) (Seed, error)
 
+		// Reset will reset the wallet, clearing the database and returning it to
+		// the unencrypted state. Reset can only be called on a wallet that has
+		// already been encrypted.
+		Reset() error
+
 		// Encrypted returns whether or not the wallet has been encrypted yet.
 		// After being encrypted for the first time, the wallet can only be
 		// unlocked using the encryption password.
 		Encrypted() bool
 
+		// InitFromSeed functions like Encrypt, but using a specified seed.
+		// Unlike Encrypt, the blockchain will be scanned to determine the
+		// seed's progress. For this reason, InitFromSeed should not be called
+		// until the blockchain is fully synced.
+		InitFromSeed(masterKey crypto.TwofishKey, seed Seed) error
+
 		// Lock deletes all keys in memory and prevents the wallet from being
 		// used to spend coins or extract keys until 'Unlock' is called.
 		Lock() error
@@ -247,6 +256,10 @@ type (
 		// derived from the master key.
 		Unlock(masterKey crypto.TwofishKey) error
 
+		// ChangeKey changes the wallet's materKey from masterKey to newKey,
+		// re-encrypting the wallet with the provided key.
+		ChangeKey(masterKey crypto.TwofishKey, newKey crypto.TwofishKey) error
+
 		// Unlocked returns true if the wallet is currently unlocked, false
 		// otherwise.
 		Unlocked() bool
@@ -267,15 +280,6 @@ type (
 		// public keys generated by any of the seeds returned.
 		AllSeeds() ([]Seed, error)
 
-		// PrimarySeed returns the current primary seed of the wallet,
-		// unencrypted, with an int indicating how many addresses have been
-		// consumed.
-		PrimarySeed() (Seed, uint64, error)
-
-		// NextAddress returns a new coin addresses generated from the
-		// primary seed.
-		NextAddress() (types.UnlockConditions, error)
-
 		// CreateBackup will create a backup of the wallet at the provided
 		// filepath. The backup will have all seeds and keys.
 		CreateBackup(string) error
@@ -299,6 +303,22 @@ type (
 		// and will have the siag keys loaded into the wallet so that they will
 		// become spendable.
 		LoadSiagKeys(crypto.TwofishKey, []string) error
+
+		// NextAddress returns a new coin addresses generated from the
+		// primary seed.
+		NextAddress() (types.UnlockConditions, error)
+
+		// PrimarySeed returns the unencrypted primary seed of the wallet,
+		// along with a uint64 indicating how many addresses may be safely
+		// generated from the seed.
+		PrimarySeed() (Seed, uint64, error)
+
+		// SweepSeed scans the blockchain for outputs generated from seed and
+		// creates a transaction that transfers them to the wallet. Note that
+		// this incurs a transaction fee. It returns the total value of the
+		// outputs, minus the fee. If only siafunds were found, the fee is
+		// deducted from the wallet.
+		SweepSeed(seed Seed) (coins, funds types.Currency, err error)
 	}
 
 	// Wallet stores and manages siacoins and siafunds. The wallet file is
@@ -349,6 +369,10 @@ type (
 		// a TransactionBuilder which can be used to expand the transaction.
 		RegisterTransaction(t types.Transaction, parents []types.Transaction) TransactionBuilder
 
+		// Rescanning reports whether the wallet is currently rescanning the
+		// blockchain.
+		Rescanning() bool
+
 		// StartTransaction is a convenience method that calls
 		// RegisterTransaction(types.Transaction{}, nil)
 		StartTransaction() TransactionBuilder
@@ -359,6 +383,9 @@ type (
 		// are also returned to the caller.
 		SendSiacoins(amount types.Currency, dest types.UnlockHash) ([]types.Transaction, error)
 
+		// SendSiacoinsMulti sends coins to multiple addresses.
+		SendSiacoinsMulti(outputs []types.SiacoinOutput) ([]types.Transaction, error)
+
 		// SendSiafunds is a tool for sending siafunds from the wallet to an
 		// address. Sending money usually results in multiple transactions. The
 		// transactions are automatically given to the transaction pool, and
